@@ -4,9 +4,9 @@ import argparse
 def version_impl(called_by_app, desired_version):
     import os
 
-    from ...bridge.app import get_application
-    from ...metadata.core import ProjectMetadata
-    from ...plugin.manager import PluginManager
+    from hatchling.bridge.app import get_application
+    from hatchling.metadata.core import ProjectMetadata
+    from hatchling.plugin.manager import PluginManager
 
     app = get_application(called_by_app)
 
@@ -33,8 +33,8 @@ def version_impl(called_by_app, desired_version):
     updated_version = metadata.hatch.version.scheme.update(desired_version, original_version, version_data)
     source.set_version(updated_version, version_data)
 
-    app.display_info('Old: {}'.format(original_version))
-    app.display_info('New: {}'.format(updated_version))
+    app.display_info(f'Old: {original_version}')
+    app.display_info(f'New: {updated_version}')
 
 
 def version_command(subparsers, defaults):
