@@ -12,7 +12,7 @@ except ImportError:
 
 
 def load_toml(path):
-    with open(path, 'r', encoding='utf-8') as f:
+    with open(path, encoding='utf-8') as f:
         return tomllib.loads(f.read())
 
 
@@ -251,6 +251,7 @@ class CoreMetadata:
         self.hatch_metadata = hatch_metadata
         self.context = context
 
+        self._raw_name = None
         self._name = None
         self._version = None
         self._description = None
@@ -279,30 +280,40 @@ class CoreMetadata:
         self._dynamic = None
 
     @property
-    def name(self):
+    def raw_name(self):
         """
         https://peps.python.org/pep-0621/#name
         """
-        if self._name is None:
+        if self._raw_name is None:
             if 'name' in self.dynamic:
                 raise ValueError('Static metadata field `name` cannot be present in field `project.dynamic`')
             elif 'name' in self.config:
-                name = self.config['name']
+                raw_name = self.config['name']
             else:
-                name = ''
+                raw_name = ''
 
-            if not name:
+            if not raw_name:
                 raise ValueError('Missing required field `project.name`')
-            elif not isinstance(name, str):
+            elif not isinstance(raw_name, str):
                 raise TypeError('Field `project.name` must be a string')
 
-            if not is_valid_project_name(name):
+            if not is_valid_project_name(raw_name):
                 raise ValueError(
                     'Required field `project.name` must only contain ASCII letters/digits, underscores, '
                     'hyphens, and periods, and must begin and end with ASCII letters/digits.'
                 )
 
-            self._name = normalize_project_name(name)
+            self._raw_name = raw_name
+
+        return self._raw_name
+
+    @property
+    def name(self):
+        """
+        https://peps.python.org/pep-0621/#name
+        """
+        if self._name is None:
+            self._name = normalize_project_name(self.raw_name)
 
         return self._name
 
@@ -393,7 +404,7 @@ class CoreMetadata:
                 if not os.path.isfile(readme_path):
                     raise OSError(f'Readme file does not exist: {readme}')
 
-                with open(readme_path, 'r', encoding='utf-8') as f:
+                with open(readme_path, encoding='utf-8') as f:
                     self._readme = f.read()
 
                 self._readme_content_type = content_type
@@ -422,7 +433,7 @@ class CoreMetadata:
                     if not os.path.isfile(path):
                         raise OSError(f'Readme file does not exist: {relative_path}')
 
-                    with open(path, 'r', encoding=readme.get('charset', 'utf-8')) as f:
+                    with open(path, encoding=readme.get('charset', 'utf-8')) as f:
                         contents = f.read()
 
                     readme_path = relative_path
@@ -539,7 +550,7 @@ class CoreMetadata:
                     if not os.path.isfile(path):
                         raise OSError(f'License file does not exist: {relative_path}')
 
-                    with open(path, 'r', encoding='utf-8') as f:
+                    with open(path, encoding='utf-8') as f:
                         contents = f.read()
                 elif 'text' in data:
                     contents = data['text']
@@ -788,6 +799,8 @@ class CoreMetadata:
         https://peps.python.org/pep-0621/#classifiers
         """
         if self._classifiers is None:
+            from hatchling.metadata.classifiers import KNOWN_CLASSIFIERS, SORTED_CLASSIFIERS, is_private
+
             if 'classifiers' in self.config:
                 classifiers = self.config['classifiers']
                 if 'classifiers' in self.dynamic:
@@ -806,10 +819,12 @@ class CoreMetadata:
             for i, classifier in enumerate(classifiers, 1):
                 if not isinstance(classifier, str):
                     raise TypeError(f'Classifier #{i} of field `project.classifiers` must be a string')
+                elif not is_private(classifier) and classifier not in KNOWN_CLASSIFIERS:
+                    raise ValueError(f'Unknown classifier in field `project.classifiers`: {classifier}')
 
                 unique_classifiers.add(classifier)
 
-            self._classifiers = sorted(unique_classifiers)
+            self._classifiers = sorted(unique_classifiers, key=lambda c: is_private(c) or SORTED_CLASSIFIERS.index(c))
 
         return self._classifiers
 
@@ -833,7 +848,7 @@ class CoreMetadata:
 
             sorted_urls = {}
 
-            for label, url in sorted(urls.items()):
+            for label, url in urls.items():
                 if not isinstance(url, str):
                     raise TypeError(f'URL `{label}` of field `project.urls` must be a string')
 
@@ -988,7 +1003,8 @@ class CoreMetadata:
                 else:
                     if requirement.url and not self.hatch_metadata.allow_direct_references:
                         raise ValueError(
-                            f'Dependency #{i} of field `project.dependencies` cannot be a direct reference'
+                            f'Dependency #{i} of field `project.dependencies` cannot be a direct reference unless '
+                            f'field `tool.hatch.metadata.allow-direct-references` is set to `true`'
                         )
 
                     dependencies_complex[get_normalized_dependency(requirement)] = requirement
@@ -1063,7 +1079,8 @@ class CoreMetadata:
                         if requirement.url and not self.hatch_metadata.allow_direct_references:
                             raise ValueError(
                                 f'Dependency #{i} of option `{option}` of field `project.optional-dependencies` '
-                                f'cannot be a direct reference'
+                                f'cannot be a direct reference unless field '
+                                f'`tool.hatch.metadata.allow-direct-references` is set to `true`'
                             )
 
                         entries[get_normalized_dependency(requirement)] = requirement
